= dl.so

# Ruby/DL provides an interface to the dynamic linker such as dlopen() on UNIX
# and LoadLibrary() on Windows.

Ruby/DL ϡUNIX  ((<dlopen(3)|manual page>))  Windows  
(({LoadLibrary()})) ʤɤΥʥߥå󥫤ؤΥ󥿥ե󶡤
롣

# = Building and Installing
# 
#   $ ruby extconf.rb    # to create the Makefile
#   $ make               # to build the library 'dl.so'
#   $ make libtest.so    # to build the C library 'libtest.so' for the test script
#   $ make test          # to run the test script
#   $ make install       # to install the library
#   $ make clean         # to remove the created files without Makefile
#   $ make distclean     # to remove the all created files

= Using Ruby/DL

# We should usually use DL::Importable module provided by "dl/import.rb".
# It has high-level functions to access library functions. We use
# DL::Importable module to extend a module as follows:

̾ϡ((<"dl/import.rb">)) DL::Importable ⥸塼Ѥ롣
ϥ饤֥ؿ˥뤿ιδؿäƤ롣
⥸塼ĥˤϰʲΤ褦 DL::Importable Ѥ롣

  require "dl/import"
  module LIBC
    extend DL::Importable
  end

# Now we can use methods dlload and extern in this module. We load the
# libraries using dlload, and define wrapper methods to library functions
# using extern respectively as follows:

ʸ塢Υ⥸塼 dlload  extern ᥽åɤѤǤ롣ʲΤ
 dlload Ȥäƥ饤֥ɤ줾Υ饤֥ؿФ
 extern Ѥ뤳Ȥǥåѡ᥽åɤ롣

  module LIBC
    extend DL::Importable
    dlload "libc.so.6","libm.so.6"
    extern "int strlen(char*)"
  end
  # Note that we should not include the module LIBC from some reason.

# We can call the library function strlen() using LIBC.strlen. If the first
# character of given function name is an uppercase, the first character of the
# defined method name becomes lowercase.
# We can also construct memory images of structures and unions using functions
# struct and union which are defined in "dl/struct.rb" as follows:

LIBC.strlen Ѥ뤳Ȥǡ饤֥ؿ strlen() ѤǤ롣Ϳ
줿ؿ̾κǽʸʸʤ顢᥽å̾κǽʸ
Ͼʸˤʤ롣

ʲΤ褦 ((<"dl/struct.rb">))  struct  union ؿѤ
뤳Ȥǹ¤Τ䶦ΤΥꥤ᡼뤳ȤǤ롣

  require "dl/import"
  require "dl/struct"
  module LIBC
    extend DL::Importable
    Timeval = struct [       # define timeval structure.
      "long tv_sec",
      "long tv_uses",
    ]
  end
  val = LIBC::Timeval.malloc # allocate memory.

# Notice that the above example takes LIBC::Timeval.malloc to allocate memory,
# rather than LIBC::Timeval.new. It is because DL::Timeval.new is for wrapping
# an object, PtrData, which has already been created.

ǡγƤ LIBC::Timeval.new ǤϤʤ
LIBC::Timeval.malloc ѤƤ뤳ȤաLIBC::Timeval.new ϡ
Ѥߤ PtrData ֥Ȥåפ뤿ΤΤ

# We can define a callback using the module function "callback" as follows:

ʲΤ褦 ⥸塼ؿ callback ѤХåǤ롣

  module Foo
    extend DL::Importable
    def my_comp(str1,str2)
      str1 <=> str2
    end
    COMPARE = callback "int my_comp(char*,char*)"
  end

# where Foo::COMPARE is a Symbol object which invokes the method "my_comp".

 Foo::COMPARE ϡmy_comp ᥽åɤư Symbol ֥Ȥ

# DL::Importable module is very useful. However, we sometimes encounter a case
# that we must directly use low-level functions such as dlsym(). In such case,
# we would use DL module functions. They are described in next section.

DL::Importable ⥸塼ϤȤƤȤˤdlsym() Τ
٥ؿľܻȤʤФʤʤ̤˽Ф魯Τ褦ʾ
ˤ DL ⥸塼δؿѤ뤳ȤˤʤˤĤƤϼ
롣

= DL module

# Module DL consists of three classes, a few module functions and constants.
# The class Symbol represents the symbol we can call. The class PtrData
# indicates a memory block such as a pointer in C. An object instantiated from
# the class Handle keeps a handle to opened library.

⥸塼 DL ϿĤΥ⥸塼ؿ 3 ĤΥ饹
Ƥ롣饹 Symbol ϸƤӽФǤ륷ܥ롣饹 
PtrData ϡC Υݥ󥿤Τ褦ʥ֥å򼨤饹 Handle 
β줿֥Ȥϥץ󤷤饤֥ġ

== Constants

* RTLD_GLOBAL
* RTLD_LAZY
* RTLD_NOW
* MAX_ARG
* MAX_CBARG
* MAX_CBENT

== Functions

* handle = dlopen(lib){|handle| ... }
#    * is quite equal to `Handle.new(lib)'
   * `Handle.new(lib)' 

* sym = set_callback(cbtype, entry){|args| ... }
* sym = set_callback(cbtype, entry, proc)
#   * makes entry-th pre-defined function to call the proc or given block. the 
#     entry-th pre-defined function is specified by cbtype and entry. cbtype is a
#     prototype of the callback. see also the section `Type specifiers' about 
#     cbtype.
  * proc ꤷ֥åƤ entry-th pre-defined function 
    롣entry-th pre-defined function  cbtype  entry ǻꤵ
    롣cbtype ϥХåΥץȥסcbtype ˤĤƤ `Type
    specifiers' ⻲ȡ

* sym = get_callback(cbtype, entry)
#   * returns the Proc object which is given by the above function
#    `set_callback'.
  * 嵭 `set_callback' ؿˤäͿ줿 Proc ֥Ȥ֤

* ptr = malloc(size, [free = nil])
#   * allocates the size bytes, and returns the pointer as a PtrData object ptr.
  * size ХȤΥΰơPtrData ֥Ȥ ptr Ȥ
    ֤

* ptr = strdup(str)
#   * returns a PtrData object ptr which represents the pointer to a new string
#     which is a duplicate of the string str.
  * ʸ str ʣʸؤΥݥ󥿤 PtrData 
    ֥Ȥ֤

* size = sizeof(type)
#   * returns the size of type. `sizeof("C") + sizeof("L")' is not equal to
#     `sizeof("CL")'. the latter is assumed to returns the enough size of the
#     structure `struct foo { char c; long l; }', but the size may not equal to
#     `sizeof(foo)' of C.
  * Υ֤`sizeof("C") + sizeof("L")' ϡ
    `sizeof("CL")' ǤϤʤԤϹ¤ `struct foo { char c;
    long l; }' νʬʥ֤ȿꤵ뤬Υ C  
    `sizeof(foo)' ƱǤϤʤ⤷ʤ

    (: sizeof ᥽åɤȼ˥饤Ȥθƥ
    뤬 C ΤȤϰۤʤ뤫⤷ʤȤȤäƤ)

    (: sizeof("L3") Τ褦˷θ˿դ뤳ȤǤ롣
    Ǥ뷿ˤĤƤ ((<dl.so/Type specifiers>)) 򻲾)

== Handle class

* handle = Handle.new(lib){|handle| ... }
#   * opens a library lib and returns a Handle object handle. if a block is
#     given, the handle is automatically closed as the block ends.

  * 饤֥ lib 򥪡ץ󤷡Handle ֥ handle ֤
    åꤹС(: ץ󤷤ϥɥ֥å
    ƥ֥å¹Ԥ)ϥɥϥ֥åνǼưŪ˥
    롣

* Handle#close
#   * closes the handle opened by the above Handle.new(lib).

  * 嵭 Handle.new(lib) ˤäƥץ󤵤줿ϥɥ򥯥롣

* sym = Handle#sym(func, prototype = "0"),
  sym = Handle#[func, prototype = nil]

#   * obtains the pointer to a function called func and returns a Symbol object
#     or a DataPtr object. prototype is a string which consists of type
#     specifiers, it indicates the function's prototype. see also the section
#     `Type specifiers'.

  * ؿ func (:䥰Хѿ)ؤΥݥ󥿤Symbol 
    ֥Ȥ DataPtr ֥Ȥ֤prototype Ϸ
    ʸǴؿΥץȥפ򼨤((<dl.so/Type specifiers>)) 
    ȡ

== Symbol class

* sym = Symbol.new(addr, type = nil, name = nil)
#   * creates the Symbol object sym with the type type if type is not nil. addr
#     is the address where the function is allocated. If type is nil, it returns
#     a DataPtr object.

  * type  nil Ǥʤʤ  type  Symbol ֥ sym 롣
    ؿƤƤʤ addr ϤΥɥ쥹type  nil ʤ
    DataPtr ֥Ȥ֤

* Symbol::char2type(char)
#   * takes a character char that represents a type and returns the type
#     specifier of the C language.
  * ʸ char ꡢC η֤

* str = Symbol#proto()
#   * returns the function prototype.
  * ؿΥץȥפ֤

* str = Symbol#name()
#   * Returns the function name.
  * ؿ֤̾

* str = Symbol#cproto(),
  str = Symbol#to_s()
#   * returns the prototype of the C language.
  * C Υץȥפ֤

* str = Symbol#inspect()
#   * returns the inspectable string.
  * ʹ֤ɤߤ䤹ʸ֤(:)

* r,rs = Symbol#call(arg1,arg2,...,argN),
  r,rs = Symbol#[](arg1,arg2,...,argN)
#   * calls the function with parameters arg1, arg2, ..., argN. and the result
#     consists of the return value r and parameters rs. rs is an array.
  * ѥ᡼ arg1, arg2, ... argN ǴؿƤӽФ̤ϡ r 
    ѥ᡼rs ʤ롣rs 

* ptr = Symbol#to_ptr
#   * returns the corresponding PtrData object ptr.
  * б PtrData ֥ ptr ֤

== PtrData class

* ptr = PtrData.new(addr, [size = 0, free = nil])
#   * returns the PtrData object representing the pointer which indicates the
#     address addr. GC frees the memory using the free function.

  * ɥ쥹 addr ؤݥ󥿤 PtrData ֥Ȥ֤
    GC  free ؿѤƥ롣

* PtrData#free=(sym)
#   * If you specify a symbol object sym, GC frees the memory using the function
#     represented by sym.

  * ܥ륪֥ sym ꤷ硢GC  sym ؿ
    Ȥäƥ롣

* sym = PtrData#free
#   * returns a symbol object sym which is used when GC frees the memory. it
#     usually configured by `PtrData#free=' or `PtrData.new'.

  * GC Τ˻Ѥ륷ܥ륪֥ sym ֤
    sym  `PtrData#free='  `PtrData.new' ˤäꤵ롣

* size = PtrData#size, PtrData#size=(size)
#   * gets and sets allocated size of the memory.
  *  size Ƥꤹ롣

* ary = PtrData#to_a(type, [size])
#   * returns an array of the type which specified with type. type must be one of
#     'S','P','I','L','D' and 'F'.

  * type ǻꤵ줿֤type  'S','P','I','L','D' 
     'F' Τ줫

* str = PtrData#to_s([len])
#   * returns a string which length is len. if len is omitted, the end of the
#     string is '\0'.

  * Ĺ len ʸ֤len άϡʸν '\0'

* ptr = PtrData#ptr,+@
#   * returns the pointed value as a PtrData object ptr.

  * ݥ󥿤ؤͤ PtrData ֥ ptr ֤(:)
    (:PtrDataؤͤݥ󥿤ǤȤơPtrData ˤ֤())

* ptr = PtrData#ref,-@
#   * returns the reference as a PtrData object ptr.

  * Ȥ PtrData ֥ ptr ֤(:)
    (:PtrDataؤλȤPtrData ˤ֤())
    (:PtrDataؤݥ󥿤PtrData ˤ֤())

* ptr = PtrData#+
#   * returns the PtrData object

  * PtrData ֥Ȥ֤(: ХȤ­ 
    PtrData ֥Ȥ֤)

* ptr = PtrData#-
#   * returns the PtrData object

  * PtrData ֥Ȥ֤(: ХȤ 
    PtrData ֥Ȥ֤)

* PtrData#struct!(type, *members)
#   * defines the data type to get access to a structure member with a symbol.
#     (see also PtrData#[])

  * ¤ΥФ򥷥ܥǥǡ(PtrData#[]
    ⻲)

* PtrData#union!(type, *members)
#   * defines the data type to get access to a union member with a symbol. (see
#     also PtrData#[])

  * ΥФ򥷥ܥǥǡ(PtrData#[]
    ⻲)

* val = PtrData#[key], PtrData#[key, num = 0]
#   * if the key is a string or symbol, this method returns the value of the
#     structure/union member which has the type defined by PtrData#
#     {struct!,union!}. if the key is a integer value and this object represents
#     the pointer ptr, it returns the value of `(ptr + key).to_s(num)'

  * key ʸ䥷ܥʤ顢Υ᥽åɤϹ¤/ΥФ
    ֤ PtrData#{struct!,union!} ˤä줿ġ
    key ǤΥ֥Ȥݥ ptr ʤ顢`(ptr +
    key).to_s(num)' ֤ͤ

* PtrData#[key,num]=val, PtrData#[key]=val
#   * if the key is a string or symbol, this method substitute the value of the
#     structure/union member with val. if the key is a integer value and val is a
#     string, this method copies num bytes of val to the memory area ptr using
#     memcpy(3).

  * key ʸ䥷ܥʤ顢Υ᥽åɤϹ¤/ΥФ
     val ˤ롣key  val ʸʤ memcpy(3) Ȥäƥ
    ΰ ptr ؤͤ num ХȤ򥳥ԡ롣

== Type specifiers

# the prototype consists of the following type specifiers, first element of 
# prototype represents the type of return value, and remaining elements represent
# the type of each argument.

ץȥפϰʲηҤ롣ץȥפκǽǤ
η̣롣ĤǤϳưη̣롣

#     C : a character (char)
#     c : a pointer to a character (char *)
#     H : a short integer (short)
#     h : a pointer to a short integer (short *)
#     I : an integer (char, short, int)
#     i : a pointer to an integer (char *, short *, int *)
#     L : a long integer (long)
#     l : a pointer to a long integer (long *)
#     F : a real (float)
#     f : a pointer to a real (float *)
#     D : a real (double)
#     d : a pointer to a real (double *)
#     S : an immutable string (const char *)
#     s : a mutable string (char *)
#     A : an array (const type[])
#     a : a mutable array (type[])
#     P : a pointer (void *)
#     p : a mutable object (void *)
#     0 : void function (this must be a first character of the prototype)

    C : ʸ (char)
    c : ʸؤݥ (char *)
    H : short   (short)
    h : short ؤݥ (short *)
    I :  (char, short, int)
    i : ؤݥ (char *, short *, int *)
    L : long  (long)
    l : long ؤΥݥ (long *)
    F : ¿ (float)
    f : ¿ؤΥݥ (float *)
    D : ¿ (double)
    d : ¿ؤΥݥ(double *)
    S : (immutable)ʸ (const char *)
    s : ѹǽ(mutable)ʸ (char *)
    A :  (const type[])
    a : ѹǽ(mutable) (type[])
    P : ݥ (void *)
    p : ѹǽ(mutable)ʥݥ (void *)
    0 : void ؿ(ϥץȥפκǽʸǤʤФʤʤ)

# the cbtype consists of type specifiers 0, C, I, H, L, F, D, S and P.
# for example:

cbtype Ϸ 0, C, I, H, L, F, D, S  P 롣㤨:

    DL.callback('IPP'){|ptr1,ptr2|
      str1 = ptr1.ptr.to_s
      str2 = ptr2.ptr.to_s
      str1 <=> str2
    }
